/*
Copyright (c) 2006-2008, Tom Thielicke IT Solutions

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*/

/****************************************************************
**
** File name: main.cpp
**
****************************************************************/

#include <QApplication>
#include <QPlastiqueStyle>
#include <QString>
#include <QSettings>
#include <QCoreApplication>
#include <QTranslator>

#include "def/defines.h"
#include "sql/connection.h"
#include "widget/mainwindow.h"
#include "widget/illustrationdialog.h"

int main(int argc, char *argv[]) {

    QApplication app(argc, argv);

    // Set application name and domain
    // (this saves having to repeat the information
    // each time a QSettings object is created)
    app.setOrganizationName(APP_ORGANIZATION);
 	app.setOrganizationDomain(APP_URL);
    app.setApplicationName(APP_NAME_INTERN);

	// Settings to get and set general settings
	#if APP_PORTABLE
	QSettings settings(QSettings::IniFormat, QSettings::UserScope,
		"portable", "settings");
    settings.setPath(QSettings::IniFormat, QSettings::UserScope,
    	QCoreApplication::applicationDirPath());
	#else
	QSettings settings;
	#endif

	// Read/write language, license key and show illustration flag
	settings.beginGroup("general");
	#if APP_MAC
		bool useNativeStyle = settings.value("check_native_style", false).toBool();
	#else
		bool useNativeStyle = false;
	#endif
	QString language = settings.value("language",
		QLocale::system().name()).toString();
	settings.setValue("language", language);
	QString licenseKey = settings.value("licensekey", "").toString();
	bool showIllustration = settings.value("check_illustration", true).toBool();
	settings.endGroup();

    // Set windows style
	if (!useNativeStyle) {
	    app.setStyle("plastique");
   }

	// Translation
	// Common qt widgets
	QTranslator translatorQt;
	translatorQt.load("qt_de.qm", ":/languages/");
	app.installTranslator(&translatorQt);

	// Content (own translation files)
	/*QTranslator translatorContent;
	translatorContent.load("tipp10_en.qm", ":/languages/");
	app.installTranslator(&translatorContent);*/

	// Set path to the db if arg parameter is set
	if (argc > 1) {
		settings.beginGroup("database");
		settings.setValue("path", QString::fromLatin1(argv[1]) + "/" +
			APP_USER_DB);
		settings.endGroup();
	}

	// Create database connection
    if (!createConnection()) {
		// Cannot find or open database
		// -> exit program
		return 1;
	}

	// Show illustration widget at the beginning if not disabled by the user
	if (showIllustration) {
		IllustrationDialog illustrationDialog("illustration.png", 560, 400,
			"Einfhrung", "", "check_illustration", 0);
		illustrationDialog.exec();
	}

	// Create main window object
    MainWindow window;
    window.show();

    // Start the event loop
    return app.exec();
}
